package com.foxit.general.activity;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.os.Environment;
import android.view.KeyEvent;
import android.view.View;
import android.view.WindowManager;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.ListAdapter;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.widget.TextView;
import android.widget.Toast;

//An activity call that the project start with, which give the navigation list for directory and PDF pages
public class MainActivity extends Activity
{
	List<HashMap<String, Object>> listData	  = null;
	ListView	                  fileList	      = null;
	private boolean	              datalistisEmpty	= true;

	@Override
	//Project starts here
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.mainbrowser);	
		getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN,
		        WindowManager.LayoutParams.FLAG_FULLSCREEN);
		fileList = (ListView) findViewById(R.id.listFile);
		TextView textView = (TextView) findViewById(R.id.fileName);
		fileList.requestChildFocus(textView, fileList);
		fileList.setSelected(true);

		if (Environment.getExternalStorageState().equals(
				Environment.MEDIA_MOUNTED)) {
			inflateListView(Environment.getExternalStorageDirectory().getPath());
		} else {
			Toast.makeText(MainActivity.this,
					"Warning: External storage directory dose not mounted!",
					Toast.LENGTH_LONG).show();
			finish();
		}
		fileList.setOnItemClickListener(new MyItemOnClick());
	}

	/** 
  	 * brief inflate  data in the current path into list. <br>
  	 * 
	 * @param path					String
	 * 
	 * @return void <br>
	 */	
	//Populate item list for a directory path 
	private void inflateList(String path)
	{
		// TODO Auto-generated method stub
		listData = new ArrayList<HashMap<String, Object>>();
		File file = new File(path);
		File[] listFiles = file.listFiles();
		HashMap<String, Object> map = new HashMap<String, Object>();
		map.put("icon", R.drawable.fm_superfolder);
		map.put("filename", "..");
		map.put("path", path);
		listData.add(map);
		if (listFiles != null)
		{
			for (File f : listFiles)
			{
				map = new HashMap<String, Object>();
				if (f.isDirectory() && !f.getName().startsWith("."))
				{
					map.put("icon", R.drawable.folder);
					map.put("filename", f.getName());
					map.put("path", f.getPath());
					listData.add(map);
				}
			}
			for (File f : listFiles)
			{
				map = new HashMap<String, Object>();
				if (f.getName().toLowerCase().endsWith(".pdf"))
				{
					map.put("icon", R.drawable.default_pdf_draw);
					map.put("filename", f.getName());
					map.put("path", f.getPath());
					listData.add(map);
				}
			}
		}

		if (listData.isEmpty())
		{
			datalistisEmpty = true;
		} else
		{
			datalistisEmpty = false;
		}
	}
	
	 /** 
  	 * brief inflate List View which show the data in the current path  <br>
	 *  
	 * @param path					String
	 * 
	 * @return void <br>
	 */	
	public void inflateListView(String path)
	{
		inflateList(path);
		if (!datalistisEmpty)
		{
			fileList.setAdapter(getListSimpleAdapter());
		}
	}

	public class MyItemOnClick implements OnItemClickListener
	{
		@Override
		public void onItemClick(AdapterView<?> parent, View view, int position,
		        long id)
		{
			HashMap<String, Object> map = listData.get(position);
			String path = (String) map.get("path");
			File file = new File(path);
			String parentpath = file.getParentFile().getPath();

			if (position == 0)
			{
				if (path.equals(Environment.getExternalStorageDirectory().getPath()))
				{
					inflateListView(Environment.getExternalStorageDirectory().getPath());
					Toast.makeText(MainActivity.this, "Top Directory",
					        Toast.LENGTH_LONG).show();
				} else
				{
					inflateListView(parentpath);
				}
			} else
			{
				if (file.isDirectory())
				{
					inflateListView(path);
				} else
				{
					Intent intent = new Intent();
					intent.putExtra("fileDir", path);
					intent.setClass(MainActivity.this, FormActivity.class);
					startActivity(intent);
				}
			}
		}
	}

	private ListAdapter getListSimpleAdapter()
	{
		// TODO Auto-generated method stub
		SimpleAdapter adapter = new SimpleAdapter(this, listData,
		        R.layout.list_menu, new String[]
		        { "icon", "filename" }, new int[]
		        { R.id.icon, R.id.fileName });
		return adapter;
	}

	@Override
	public boolean onKeyDown(int keyCode, KeyEvent event)
	{
		if (keyCode == KeyEvent.KEYCODE_BACK && event.getRepeatCount() == 0)
		{
			HashMap<String, Object> map = listData.get(0);
			String path = null;
			path = (String) map.get("path");
			File file = null;
			file = new File(path);
			if (file.getParent() != null)
			{
				String parentpath = file.getParentFile().getPath();
				if (path.equals(Environment.getExternalStorageDirectory().getPath()))
				{
					this.finish();
				} else
				{
					inflateListView(parentpath);
				}
			}
			return false;
		}
		return false;
	}

}
