package com.foxit.general.activity;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import android.content.Intent;
import android.graphics.RectF;
import android.media.AudioManager;
import android.media.ToneGenerator;
import android.net.Uri;
import android.util.Log;
import android.view.Gravity;
import android.widget.Toast;

import com.foxit.general.utils.MyFileManager;
import com.foxit.gsdk.PDFException;
import com.foxit.gsdk.pdf.PDFDocument;
import com.foxit.gsdk.pdf.PDFPage;
import com.foxit.gsdk.pdf.form.FormActionHandler;

public  class MyFormActionHandler extends FormActionHandler  {

	public static final String TAG = "MyFormActionHandler";
	public Object clientData = null;
	public boolean m_bChangeMask = false;
	public PDFPage pdfPage = null;
	public PDFDocument pdfDocument = null;
	private static FormActivity parent = null;
	
	public MyFormActionHandler(FormActivity formActivity) {
		parent = formActivity;
		// TODO Auto-generated constructor stub
	}

	public void release(Object clientData) throws PDFException
	{
		if(pdfDocument != null && pdfDocument.getHandle() != 0 && pdfPage != null && pdfPage.getHandle() != 0)
		{	
			pdfDocument.closePage(pdfPage);
			pdfDocument.close();
		}
	}

	@Override
	public void invalidateRect(Object arg0, PDFDocument arg1, int arg2,
			RectF arg3) throws PDFException {
		// TODO Auto-generated method stub	
	}
	
	public int getCurrentPageIndex(Object clientData,
			final PDFDocument document) throws PDFException
	{
		Log.i(TAG, "getCurrentPageIndex.........");
		Toast.makeText(parent, "getCurrentPageIndex" , Toast.LENGTH_SHORT).show();
		return pdfPage.getIndex();
	}

	public void setCurrentPageIndex(Object clientData,
			PDFDocument document, int pageIndex) throws PDFException
	{	
		Log.i(TAG, "setCurrentPageIndex.........");
		Toast.makeText(parent, "setCurrentPageIndex\n\npageIndex:" + pageIndex, Toast.LENGTH_SHORT).show();
		pdfPage = document.getPage(pageIndex);
	}

	public int getRotation(Object clientData,
			final PDFDocument document, int pageIndex) throws PDFException
	{
		Log.i(TAG, "getRotation.....");
		Toast.makeText(parent, "getRotationn\n\npageIndex:" + pageIndex, Toast.LENGTH_SHORT).show();
		pdfPage = document.getPage(pageIndex);
		int rotation = pdfPage.getRotation();
		return rotation;
	}

	public void executeNamedAction(Object clientData,
			final PDFDocument document, final String namedAction)
			throws PDFException
	{		
		Log.i(TAG, "setCurrentPageIndex.........");
		Toast.makeText(parent, "executeNamedAction\n\nnamedAction:" + namedAction, Toast.LENGTH_SHORT).show();
	}
	
	public void setChangeMark(Object clientData,
			final PDFDocument document, boolean changeMark) throws PDFException
	{
		if(document != null)	
			m_bChangeMask = changeMark;
	}
	
	public boolean getChangeMark(Object clientData,
			final PDFDocument document) throws PDFException
	{
		if(document != null)			
			return m_bChangeMask;
		return false;
	}

	public PDFDocument[] getOpenedDocs(Object clientData)
			throws PDFException
	{
		PDFDocument[] docs = new PDFDocument[1];
		docs[0] = parent.pdfDocument;
		return docs;
	}
	
	public int alert(Object clientData, final String msg,
			final String title, int type, int icon) throws PDFException
	{
		Log.i(TAG, "alert.....");
		Toast alert = Toast.makeText(parent, "alert..."+msg+"."+title+"."+type, Toast.LENGTH_SHORT);
		alert.setGravity(Gravity.CENTER, 0, 0);
		alert.show();
		return 0;
	}

	public void beep(Object clientData, int type) throws PDFException
	{
		Log.i(TAG, "beep.....");
		Toast.makeText(parent, "beep!!!---" + type, Toast.LENGTH_SHORT).show();
		ToneGenerator toneGenerator = new ToneGenerator(AudioManager.STREAM_SYSTEM, ToneGenerator.MAX_VOLUME);
		toneGenerator.startTone(ToneGenerator.TONE_PROP_BEEP);
	}

	public String response(Object clientData, final String question,
			final String title, final String defaultValue, final String label,
			boolean isPassword) throws PDFException
	{
		Log.i(TAG, "response.....");
		Toast.makeText(parent, "response\n\nquestion:"+question+"\ntitle:"+title+
				"\ndefaultValue:"+defaultValue + "\nlabel:" + label + "\nisPassword:" + isPassword, Toast.LENGTH_SHORT).show();
		return null;
	}

	public String getFilePath(final Object clientData,
			final PDFDocument document) throws PDFException
	{
		Log.i(TAG, "getFilePath.....");
		Toast.makeText(parent, "getFilePath", Toast.LENGTH_SHORT).show();
		return null;
	}
	
	public void mail(final Object clientData,
			final PDFDocument document, boolean bUI, final String To,
			final String Subject, final String CC, final String BCC,
			final String Msg) throws PDFException
	{
		Log.i(TAG, "mail.....");
		Toast.makeText(parent, "mail\n\nTo " + To + "\nSubject:" + Subject + "\nCC:" + CC + 
				"\nBCC:" + BCC + "\nMsg" + Msg, Toast.LENGTH_SHORT).show();
		Intent i = new Intent(Intent.ACTION_SEND);
		i.setType("message/rfc822");
		i.putExtra(Intent.EXTRA_EMAIL, To);
		i.putExtra(Intent.EXTRA_SUBJECT, Subject);
		i.putExtra(Intent.EXTRA_CC, CC);
		i.putExtra(Intent.EXTRA_BCC, BCC);
		i.putExtra(Intent.EXTRA_TEXT, Msg);
		try {
			parent.startActivity(Intent.createChooser(i, "Send mail..."));
		} catch (android.content.ActivityNotFoundException ex) {
			Toast.makeText(parent, "There are no email clients installed.",
					Toast.LENGTH_SHORT).show();
		}
	}

	public void print(Object clientData, final PDFDocument document,
			boolean isUI, int start, int end, boolean isSilent,
			boolean isShrinkToFit, boolean isPrintAsImage, boolean isReverse,
			boolean isAnnotations) throws PDFException
	{
		Log.i(TAG, "print.....");	
		Toast.makeText(parent, "Printing is unsupported now.", Toast.LENGTH_SHORT).show();	
		return ;
	}

	public void submitForm(Object clientData,
			final PDFDocument document, final byte[] formData, final String URL)
			throws PDFException
	{
		Log.i(TAG, "submitForm..."+URL);
		Toast.makeText(parent, "submitForm\n\nURL:"+URL, Toast.LENGTH_SHORT).show();
		if(!URL.contains("sdcard"))
			return ;
		try {
			File formFile = new File(URL);
			formFile.createNewFile();
			FileOutputStream fo = new FileOutputStream(formFile);
			fo.write(formData);			
			fo.close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			Log.w(TAG, "submitForm: IO exception!!!");	
		}
	}

	public void launchURL(Object clientData, final String URL)
			throws PDFException
	{
		Log.i(TAG, "launchURL.....");	
		Toast.makeText(parent, "launchURL\n\nURL:"+URL, Toast.LENGTH_SHORT).show();
		
		String addr = null;
		if(!URL.contains("http"))
			addr = "http://" + URL;
		else addr = URL;
		Uri uri = Uri.parse( addr );
		parent.startActivity( new Intent( Intent.ACTION_VIEW, uri ) );
	}

	public String browseFile(Object clientData) throws PDFException
	{
		Log.i(TAG, "browseFile.....");	
		Intent browseFile = new Intent(parent,MyFileManager.class);
		parent.startActivity(browseFile);
		return null;
	}


	
}
