
#include "../../../../../include/fsdk.h"

#include <string.h>
#include <string>
#include <stdio.h>
#include <stdlib.h>
#import "gsdkObjc.h"

typedef struct _FSDK_FILEDATA_
{
    FILE* fileStream;
    FSCRT_FILEHANDLER fileHandler;
}FSDK_FILEDATA;

static void FSDK_FileHandler_Release(FS_LPVOID clientData)
{
    if (!clientData) return;
    fclose(((FSDK_FILEDATA*)clientData)->fileStream);
    
    free((FSDK_FILEDATA*)clientData);
}

//Implementation to FSCRT_FILEHANDLER::GetSize
static FS_DWORD	FSDK_FileHandler_GetSize(FS_LPVOID clientData)
{
    //Check parameter
    if (!clientData) return 0;
    
    //Move file pointer to the end, and get file size
    long curPos = ftell(((FSDK_FILEDATA*)clientData)->fileStream);
    fseek(((FSDK_FILEDATA*)clientData)->fileStream, 0, SEEK_END);
    long size = ftell(((FSDK_FILEDATA*)clientData)->fileStream);
    fseek(((FSDK_FILEDATA*)clientData)->fileStream, curPos, SEEK_SET);
    
    return (FS_DWORD)size;
}

//Implementation to FSCRT_FILEHANDLER::ReadBlock
static FS_RESULT FSDK_FileHandler_ReadBlock(FS_LPVOID clientData, FS_DWORD offset, FS_LPVOID buffer, FS_DWORD size)
{
    //Check parameter
    if (!clientData) return FSCRT_ERRCODE_FILE;
    
    //Change file pointer to offset
    fseek(((FSDK_FILEDATA*)clientData)->fileStream, (long)offset, SEEK_SET);
    //Read data
    size_t readSize = fread(buffer, 1, size, ((FSDK_FILEDATA*)clientData)->fileStream);
    return readSize == size ? FSCRT_ERRCODE_SUCCESS : FSCRT_ERRCODE_FILE;
}

//Implementation to FSCRT_FILEHANDLER::WriteBlock
static FS_RESULT FSDK_FileHandler_WriteBlock(FS_LPVOID clientData, FS_DWORD offset, FS_LPCVOID buffer, FS_DWORD size)
{
    //Check parameter
    if (!clientData) return FSCRT_ERRCODE_FILE;
    
    //Change file pointer to offset
    fseek(((FSDK_FILEDATA*)clientData)->fileStream, (long)offset, SEEK_SET);
    //Write data
    size_t writeSize = fwrite(buffer, 1, size, ((FSDK_FILEDATA*)clientData)->fileStream);
    return writeSize == size ? FSCRT_ERRCODE_SUCCESS : FSCRT_ERRCODE_FILE;
}

//Implementation to FSCRT_FILEHANDLER::Flush
static FS_RESULT FSDK_FileHandler_Flush(FS_LPVOID clientData)
{
    //Check parameter
    if (!clientData) return FSCRT_ERRCODE_FILE;
    
    //Flush file
    fflush(((FSDK_FILEDATA*)clientData)->fileStream);
    
    return FSCRT_ERRCODE_SUCCESS;
}

//Implementation to FSCRT_FILEHANDLER::Truncate
static FS_RESULT FSDK_FileHandler_Truncate(FS_LPVOID clientData, FS_DWORD size)
{
    //Unsupported
    return FSCRT_ERRCODE_UNSUPPORTED;
}

static FSCRT_FILE OpenFile(NSString* fileName, NSString* fileMode)
{
    //Open file as stream
    FILE* fileStream = fopen((FS_LPSTR)[fileName UTF8String], (FS_LPSTR)[fileMode UTF8String]);
    if (!fileStream) return NULL;
    //Allocate file data
    FSDK_FILEDATA* fileData = (FSDK_FILEDATA*)malloc(sizeof(FSDK_FILEDATA));
    if (!fileData)
    {
        //If error, close file stream and exit
        fclose(fileStream);
        return NULL;
    }
    //Initialize file handler
    fileData->fileHandler.clientData = fileData;
    fileData->fileHandler.Release = FSDK_FileHandler_Release;
    fileData->fileHandler.GetSize = FSDK_FileHandler_GetSize;
    fileData->fileHandler.ReadBlock = FSDK_FileHandler_ReadBlock;
    fileData->fileHandler.WriteBlock = FSDK_FileHandler_WriteBlock;
    fileData->fileHandler.Flush = FSDK_FileHandler_Flush;
    fileData->fileHandler.Truncate = FSDK_FileHandler_Truncate;
    fileData->fileStream = fileStream;
    //Call FSCRT_File_Create to create Foxit GSDK file object
    FSCRT_FILE file = NULL;
    if (FSCRT_File_Create(&fileData->fileHandler, &file) == FSCRT_ERRCODE_SUCCESS)
    {
        //Success, return file object
        return file;
    }
    
    //Free file data, close file stream and exit
    free(fileData);
    fclose(fileStream);
    return NULL;
}

/*******************************************************************************/
/* Library management														  */
/*******************************************************************************/
//Extension to allocate memory buffer. Implementation to FSCRT_MEMMGRHANDLER::Alloc
static FS_LPVOID	FSDK_Alloc(FS_LPVOID clientData, FS_DWORD size)
{
	return malloc((size_t)size);
}

//Extension to reallocate memory buffer. Implementation to FSCRT_MEMMGRHANDLER::Realloc
static FS_LPVOID	FSDK_Realloc(FS_LPVOID clientData, FS_LPVOID ptr, FS_DWORD newSize)
{
	return realloc(ptr, (size_t)newSize);
}

//Extension to free memory buffer. Implementation to FSCRT_MEMMGRHANDLER::Free
static void			FSDK_Free(FS_LPVOID clientData, FS_LPVOID ptr)
{
	free(ptr);
}

//Global variables for extension manager
static FSCRT_MEMMGRHANDLER	g_MemMgrHandler = {NULL, FSDK_Alloc, FSDK_Realloc, FSDK_Free};
static FS_LPVOID			g_pGlobalBuffer = NULL;

//Default size of global buffer, 16MB
#define FSDK_GLOBALBUFFER_SIZE	(1024 * 1024 * 16)

@implementation gsdkObjc
    +(FS_RESULT) initLibrary{
        g_pGlobalBuffer = malloc(FSDK_GLOBALBUFFER_SIZE);
        if (!g_pGlobalBuffer)
        {
            return FSCRT_ERRCODE_ERROR;
        }
        return FSCRT_Library_CreateMgr(g_pGlobalBuffer, FSDK_GLOBALBUFFER_SIZE, &g_MemMgrHandler);
    }

    +(void) destroyLibrary{
        FSCRT_Library_DestroyMgr();
    }

    +(FS_RESULT)unlockLibrary: (NSString*)sn and:(NSString*)key{
        FSCRT_BSTR license_id;
        FSCRT_BStr_Init(&license_id);
        license_id.str = (FS_LPSTR)[sn UTF8String];
        license_id.len = strlen(license_id.str);
        FSCRT_BSTR unlockCode;
        unlockCode.str = (FS_LPSTR)[key UTF8String];;
        unlockCode.len = strlen(unlockCode.str);
        FS_RESULT ret = FSCRT_License_UnlockLibrary(&license_id, &unlockCode);
        return ret;
    }

    +(FS_RESULT)initPDFModule{
        return FSCRT_PDFModule_Initialize();
    }

    +(FS_RESULT)destroyPDFModule{
        return FSCRT_PDFModule_Finalize();
   }

    +(FSCRT_FILE) FSDK_OpenFile: (NSString*) fileName and: (NSString*)fileMode{
        //Check parameters, return NULL if they are invalid
        if ([fileName length] == 0 || [fileMode length] == 0) return NULL;
        return OpenFile(fileName, fileMode);
    }

    +(FS_RESULT) FSDK_CloseFile: (FSCRT_FILE) file{
        return FSCRT_File_Release(file);
    }

    +(FSCRT_DOCUMENT) FSDK_LoadDocument: (FSCRT_FILE)file and: (NSString*)password and: (FS_DWORD)flag{
        FSCRT_DOCUMENT document = NULL;
        FSCRT_BSTRC(strPassword, (FS_LPSTR)[password UTF8String]);
        FS_RESULT ret = FSPDF_Doc_StartLoadWithFlag(file, &strPassword, flag, &document, NULL);
        if (ret != FSCRT_ERRCODE_SUCCESS) return NULL;
        return document;
    }

    +(FS_RESULT) FSDK_CloseDocument: (FSCRT_DOCUMENT)document{
        return FSPDF_Doc_Close(document);
    }

    +(FSCRT_PAGE) FSDK_LoadPage: (FSCRT_DOCUMENT)document and: (FS_INT32)index{
        FSCRT_PAGE page = NULL;
        FS_RESULT ret = FSPDF_Doc_GetPage(document, index, &page);
        if (ret != FSCRT_ERRCODE_SUCCESS) return NULL;
        return page;
    }

    +(FS_RESULT) FSDK_ClosePage:(FSCRT_PAGE)page{
        return FSPDF_Page_Clear(page);
    }

    +(FS_RESULT) FSDK_ParsePage:(FSCRT_PAGE)page{
        FS_RESULT ret = FSCRT_ERRCODE_ERROR;
        FSCRT_PROGRESS parseProgress = NULL;
        ret = FSPDF_Page_StartParse(page, FSPDF_PAGEPARSEFLAG_NORMAL, &parseProgress);
        if(ret != FSCRT_ERRCODE_SUCCESS) return ret;
        ret = FSCRT_Progress_Continue(parseProgress, NULL);
        if (ret != FSCRT_ERRCODE_SUCCESS || ret != FSCRT_ERRCODE_FINISHED) return ret;
        ret = FSCRT_Progress_Release(parseProgress);
        return ret;
    }

    +(FSCRT_BITMAP) FSDK_RenderPageToBitmap:(FSCRT_PAGE)page{
        FS_FLOAT width = 0, height = 0;
        FS_INT32 bimtapType = FSCRT_BITMAPFORMAT_24BPP_BGR;
        FS_ARGB colorToFill = FSCRT_ARGB_Encode(0xff, 0xff, 0xff, 0xff);
        FS_RESULT ret = FSPDF_Page_GetSize(page, &width, &height);
        if (ret != FSCRT_ERRCODE_SUCCESS) return NULL;
        
        FS_BOOL hasTrans = 0;
        FSPDF_Page_HasTransparency(page, &hasTrans);
        if (hasTrans) {
            bimtapType = FSCRT_BITMAPFORMAT_32BPP_BGRA;
        }
        FSCRT_BITMAP bitmap;
        ret = FSCRT_Bitmap_Create((FS_INT32)width, (FS_INT32)height, bimtapType, NULL, 0, &bitmap);
        if (ret != FSCRT_ERRCODE_SUCCESS) return NULL;
        
        FSCRT_RECT rect = {0, 0, (FS_INT32)width, (FS_INT32)height};
        ret = FSCRT_Bitmap_FillRect(bitmap, colorToFill, &rect);
        if (ret != FSCRT_ERRCODE_SUCCESS)
        {
            FSCRT_Bitmap_Release(bitmap);
            return NULL;
        }
        
        FSCRT_MATRIX mt;
        ret = FSPDF_Page_GetMatrix(page, 0, 0, (FS_INT32)width, (FS_INT32)height, 0, &mt);
        if (ret != FSCRT_ERRCODE_SUCCESS)
        {
            FSCRT_Bitmap_Release(bitmap);
            return NULL;
        }
        
        //Create a renderer on the given bitmap.
        FSCRT_RENDERER renderer;
        ret = FSCRT_Renderer_CreateOnBitmap(bitmap, &renderer);
        if (ret != FSCRT_ERRCODE_SUCCESS)
        {
            FSCRT_Bitmap_Release(bitmap);
            return NULL;
        }
        //Create a render context and render a page to get a bmp later.
        FSPDF_RENDERCONTEXT rendercontext;
        ret = FSPDF_RenderContext_Create(&rendercontext);
        if (ret != FSCRT_ERRCODE_SUCCESS)
        {
             FSCRT_Bitmap_Release(bitmap);
            //Release render
            FSCRT_Renderer_Release(renderer);
            return NULL;
        }
        //Set the matrix of the given render context.
        ret = FSPDF_RenderContext_SetMatrix(rendercontext, &mt);
        if (ret != FSCRT_ERRCODE_SUCCESS)
        {
            FSCRT_Bitmap_Release(bitmap);
            //Release render
            FSCRT_Renderer_Release(renderer);
            //Release render context
            FSPDF_RenderContext_Release(rendercontext);
            return NULL;
        }
        
        //Start to render with the given render context, renderer, page to get the render progress.
        FSCRT_PROGRESS renderProgress = NULL;
        ret = FSPDF_RenderContext_StartPage(rendercontext, renderer, page,FSPDF_PAGERENDERFLAG_NORMAL, &renderProgress);
        if (ret != FSCRT_ERRCODE_SUCCESS)
        {
            FSCRT_Bitmap_Release(bitmap);
            //Release render
            FSCRT_Renderer_Release(renderer);
            //Release render context
            FSPDF_RenderContext_Release(rendercontext);
            return NULL;
        }
        
        //Continue to render if it's not completed.
        ret = FSCRT_Progress_Continue(renderProgress, NULL);
        
        if (ret == FSCRT_ERRCODE_FINISHED)
        {
            ret = FSCRT_ERRCODE_SUCCESS;
        }
    
        //Release render progress 
        FSCRT_Progress_Release(renderProgress);
        //Release render
        FSCRT_Renderer_Release(renderer);
        //Release render context
        FSPDF_RenderContext_Release(rendercontext);
        
        return bitmap;
    }

    +(FS_RESULT)FSDK_SaveImageFile:(FSCRT_BITMAP)bitmap and: (FS_INT32)type and: (NSString*)fileName{
        //Check parameters, return error if they are invalid
        if (!bitmap || [fileName length] < 1)
        {
            return FSCRT_ERRCODE_ERROR;
        }
        
        //Open image file at first
        FSCRT_FILE fsFile = NULL;
        
        NSString* fileMode = @"w+";
        fsFile = OpenFile(fileName, fileMode);
        if (fsFile == NULL) return FSCRT_ERRCODE_ERROR;
//        
//        //Create image file object
        FSCRT_IMAGEFILE imageFile = NULL;
        FS_RESULT ret = FSCRT_ImageFile_Create(fsFile, type, 1, &imageFile);
        if (ret == FSCRT_ERRCODE_SUCCESS)
        {
            //Add frame
            ret = FSCRT_ImageFile_AddFrame(imageFile, bitmap);
            if (ret != FSCRT_ERRCODE_SUCCESS) return ret;
            ret = FSCRT_ImageFile_Release(imageFile);
        }
        
        //Close image file
        FSCRT_File_Release(fsFile);
        return ret;
    }
@end



