﻿using System;

using UIKit;
using Foundation;
using ObjCRuntime;
using CoreGraphics;
using System.Runtime.InteropServices;

namespace iosBinding
{
	// The first step to creating a binding is to add your native library ("libNativeLibrary.a")
	// to the project by right-clicking (or Control-clicking) the folder containing this source
	// file and clicking "Add files..." and then simply select the native library (or libraries)
	// that you want to bind.
	//
	// When you do that, you'll notice that MonoDevelop generates a code-behind file for each
	// native library which will contain a [LinkWith] attribute. MonoDevelop auto-detects the
	// architectures that the native library supports and fills in that information for you,
	// however, it cannot auto-detect any Frameworks or other system libraries that the
	// native library may depend on, so you'll need to fill in that information yourself.
	//
	// Once you've done that, you're ready to move on to binding the API...
	//
	//
	// Here is where you'd define your API definition for the native Objective-C library.
	//
	// For example, to bind the following Objective-C class:
	//
	//     @interface Widget : NSObject {
	//     }
	//
	// The C# binding would look like this:
	//
	//     [BaseType (typeof (NSObject))]
	//     interface Widget {
	//     }
	//
	// To bind Objective-C properties, such as:
	//
	//     @property (nonatomic, readwrite, assign) CGPoint center;
	//
	// You would add a property definition in the C# interface like so:
	//
	//     [Export ("center")]
	//     CGPoint Center { get; set; }
	//
	// To bind an Objective-C method, such as:
	//
	//     -(void) doSomething:(NSObject *)object atIndex:(NSInteger)index;
	//
	// You would add a method definition to the C# interface like so:
	//
	//     [Export ("doSomething:atIndex:")]
	//     void DoSomething (NSObject object, int index);
	//
	// Objective-C "constructors" such as:
	//
	//     -(id)initWithElmo:(ElmoMuppet *)elmo;
	//
	// Can be bound as:
	//
	//     [Export ("initWithElmo:")]
	//     IntPtr Constructor (ElmoMuppet elmo);
	//
	// For more information, see http://developer.xamarin.com/guides/ios/advanced_topics/binding_objective-c/
	//
	[BaseType (typeof (NSObject))]
	interface gsdkObjc
	{
		[Static, Export("initLibrary")]
		int initLibrary();

		[Static, Export("destroyLibrary")]
		void destroyLibrary();

		[Static, Export("unlockLibrary:and:")]
		int unlockLibrary([MarshalAs(UnmanagedType.LPStr)]string sn, [MarshalAs(UnmanagedType.LPStr)]string key);

		[Static, Export("initPDFModule")]
		int initPDFModule();

		[Static, Export("destroyPDFModule")]
		int destroyPDFModule();

		[Static, Export("FSDK_OpenFile:and:")]
		IntPtr FSDK_OpenFile([MarshalAs(UnmanagedType.LPStr)]string fileName, [MarshalAs(UnmanagedType.LPStr)]string fileMode);

		[Static, Export("FSDK_CloseFile:")]
		int FSDK_CloseFile(IntPtr file);

		[Static, Export("FSDK_LoadDocument:and:and:")]
		IntPtr FSDK_LoadDocument(IntPtr file, [MarshalAs(UnmanagedType.LPStr)]string password, int flag);

		[Static, Export("FSDK_CloseDocument:")]
		int FSDK_CloseDocument(IntPtr document);

		[Static, Export("FSDK_LoadPage:and:")]
		IntPtr FSDK_LoadPage(IntPtr document, int index);

		[Static, Export("FSDK_ClosePage:")]
		int FSDK_ClosePage(IntPtr page);

		[Static, Export("FSDK_ParsePage:")]
		int FSDK_ParsePage(IntPtr page);

		[Static, Export("FSDK_RenderPageToBitmap:")]
		IntPtr FSDK_RenderPageToBitmap(IntPtr page);

		[Static, Export("FSDK_SaveImageFile:and:and:")]
		int FSDK_SaveImageFile(IntPtr bitmap, int type, [MarshalAs(UnmanagedType.LPStr)]string fileName);
	}
}

